from __future__ import with_statement
import errno
import os
import shutil
import StringIO

from flask import current_app, url_for
from .base import Storage, StorageFile, StorageException, reraise as _reraise


def reraise(exception):
    if exception.errno == errno.EEXIST:
        exception.status = 409
    elif exception.errno == errno.ENOENT:
        exception.status = 404
    exception.message = exception.strerror
    _reraise(exception)


class FileSystemStorage(Storage):
    """
    Standard filesystem storage
    """

    def __init__(self, folder_name=None, file_view=None):
        if folder_name is None:
            folder_name = current_app.config.get(
                'UPLOADS_FOLDER',
                os.path.dirname(__file__)
            )
        if file_view is None:
            file_view = current_app.config.get(
                'FILE_SYSTEM_STORAGE_FILE_VIEW',
                'uploads.uploaded_file'
            )
        self._folder_name = folder_name
        self._file_view = file_view
        self._absolute_path = os.path.abspath(folder_name)

    @property
    def folder_name(self):
        return self._folder_name

    def list_folders(self):
        if not self._absolute_path:
            raise StorageException('No folder given in class constructor.')
        return filter(
            lambda a: os.path.isdir(os.path.join(self._absolute_path, a)),
            os.listdir(self._absolute_path)
        )

    def list_files(self):
        if not self._absolute_path:
            raise StorageException('No folder given in class constructor.')
        return filter(
            lambda a: not os.path.isdir(os.path.join(self._absolute_path, a)),
            os.listdir(self._absolute_path)
        )

    def _save(self, name, content):
        full_path = self.path(name)
        directory = os.path.dirname(full_path)
        try:
            self.create_folder(directory)
        except StorageException, e:
            if e.status_code != 409:
                raise e

        with open(full_path, 'wb') as destination:
            buffer_size = 16384
            # we should allow strings to be passed as content since the other
            # drivers support this too
            if isinstance(content, basestring):
                io = StringIO.StringIO()
                io.write(content)
                content = io

            content.seek(0)
            try:
                shutil.copyfileobj(content, destination, buffer_size)
            except OSError, e:
                reraise(e)
        return self.file_class(self, name)

    def open(self, name, mode='rb'):
        try:
            file_ = self.file_class(self, name)
            file_.file
            return file_
        except IOError, e:
            reraise(e)

    def delete_folder(self, name):
        path = self.path(name)
        try:
            return shutil.rmtree(path)
        except OSError, e:
            reraise(e)

    def create_folder(self, path):
        try:
            return os.makedirs(path)
        except OSError, e:
            reraise(e)

    def delete(self, name):
        name = self.path(name)
        try:
            return os.remove(name)
        except OSError, e:
            reraise(e)

    def exists(self, name):
        return os.path.exists(self.path(name))

    def path(self, name):
        return os.path.normpath(os.path.join(self._absolute_path, name))

    def url(self, name):
        return url_for(self._file_view, filename=name)

    @property
    def file_class(self):
        return FileSystemStorageFile


class FileSystemStorageFile(StorageFile):
    _file = None

    def __init__(self, storage, name=None, prefix=''):
        self._storage = storage
        if name is not None:
            self.name = name
        self.prefix = prefix

    @property
    def file(self):
        if not self._file:
            self._file = open(self.path, 'rb')
        return self._file

    @property
    def path(self):
        return self._storage.path(self.name)

    @property
    def last_modified(self):
        return os.path.getmtime(self.path)

    @property
    def size(self):
        return os.path.getsize(self.path)

    @property
    def url(self):
        return self._storage.url(self.name)

    def tell(self):
        return self.file.tell()

    def read(self, size=-1):
        return self.file.read(size)

    def seek(self, offset, whence=os.SEEK_SET):
        self.file.seek(offset, whence)



from functools import wraps
import mimetypes

from boto.s3.connection import S3Connection, SubdomainCallingFormat
from boto.exception import S3ResponseError, S3CreateError
from boto.s3.key import Key

from flask import current_app

from .base import (
    FileNotFoundError,
    Storage,
    StorageException,
    StorageFile,
    reraise
)


class S3BotoStorage(Storage):
    def __init__(
            self,
            folder_name=None,
            access_key=None,
            secret_key=None,
            bucket_acl=None,
            acl=None,
            headers=None,
            gzip=None,
            gzip_content_types=None,
            querystring_auth=None,
            querystring_expire=None,
            reduced_redundancy=None,
            custom_domain=None,
            secure_urls=None,
            location=None,
            file_name_charset=None,
            preload_metadata=None,
            calling_format=None,
            file_overwrite=None,
            auto_create_bucket=None):

        self.access_key = access_key or \
            current_app.config.get('AWS_ACCESS_KEY_ID', None)
        self.secret_key = secret_key or \
            current_app.config.get('AWS_SECRET_ACCESS_KEY', None)
        self.calling_format = calling_format or \
            current_app.config.get(
                'AWS_S3_CALLING_FORMAT',
                SubdomainCallingFormat()
            )
        self.auto_create_bucket = auto_create_bucket or \
            current_app.config.get('AWS_AUTO_CREATE_BUCKET', False)
        self.bucket_name = folder_name or \
            current_app.config.get('AWS_STORAGE_BUCKET_NAME', None)
        self.acl = acl or \
            current_app.config.get('AWS_DEFAULT_ACL', 'public-read')
        self.bucket_acl = bucket_acl or \
            current_app.config.get('AWS_BUCKET_ACL', self.acl)
        self.file_overwrite = file_overwrite or \
            current_app.config.get('AWS_S3_FILE_OVERWRITE', False)
        self.headers = headers or \
            current_app.config.get('AWS_HEADERS', {})
        self.preload_metadata = preload_metadata or \
            current_app.config.get('AWS_PRELOAD_METADATA', False)
        self.gzip = gzip or \
            current_app.config.get('AWS_IS_GZIPPED', False)
        self.gzip_content_types = gzip_content_types or \
            current_app.config.get(
                'GZIP_CONTENT_TYPES', (
                    'text/css',
                    'application/javascript',
                    'application/x-javascript',
                )
            )
        self.querystring_auth = querystring_auth or \
            current_app.config.get('AWS_QUERYSTRING_AUTH', True)
        self.querystring_expire = querystring_expire or \
            current_app.config.get('AWS_QUERYSTRING_EXPIRE', 3600)
        self.reduced_redundancy = reduced_redundancy or \
            current_app.config.get('AWS_REDUCED_REDUNDANCY', False)
        self.custom_domain = custom_domain or \
            current_app.config.get('AWS_S3_CUSTOM_DOMAIN', None)
        self.secure_urls = secure_urls or \
            current_app.config.get('AWS_S3_SECURE_URLS', True)
        self.location = location or current_app.config.get('AWS_LOCATION', '')
        self.location = self.location.lstrip('/')
        self.file_name_charset = file_name_charset or \
            current_app.config.get('AWS_S3_FILE_NAME_CHARSET', 'utf-8')

        self._connection = None
        self._entries = {}

    @property
    def connection(self):
        if self._connection is None:
            self._connection = S3Connection(
                self.access_key, self.secret_key,
                calling_format=self.calling_format
            )
        return self._connection

    @property
    def folder_name(self):
        return self.bucket_name

    @property
    def bucket(self):
        """
        Get the current bucket. If there is no current bucket object
        create it.
        """
        if not hasattr(self, '_bucket'):
            self._bucket = self._get_or_create_bucket(self.bucket_name)
        return self._bucket

    def list_folders(self):
        return [bucket.name for bucket in self.connection.get_all_buckets()]

    @property
    def folder(self):
        return self.bucket

    def list_files(self):
        return [
            self.file_class(self, key.name) for key in self.bucket.list()
        ]

    def create_folder(self, name=None):
        if not name:
            name = self.folder_name
        try:
            bucket = self.connection.create_bucket(name)
            bucket.set_acl(self.bucket_acl)
        except S3CreateError, e:
            reraise(e)
        return bucket

    def _get_or_create_bucket(self, name):
        """Retrieves a bucket if it exists, otherwise creates it."""
        try:
            return self.connection.get_bucket(
                name,
                validate=self.auto_create_bucket
            )
        except S3ResponseError:
            if self.auto_create_bucket:
                bucket = self.connection.create_bucket(name)
                bucket.set_acl(self.bucket_acl)
                return bucket
            raise RuntimeError(
                "Bucket specified by "
                "S3_BUCKET_NAME does not exist. "
                "Buckets can be automatically created by setting "
                "AWS_AUTO_CREATE_BUCKET=True")

    def _save(self, name, content):
        cleaned_name = self._clean_name(name)
        name = self._normalize_name(cleaned_name)
        headers = self.headers.copy()
        name = cleaned_name
        content_type = mimetypes.guess_type(name)[0] or Key.DefaultContentType
        encoded_name = self._encode_name(name)

        key = self.bucket.new_key(encoded_name)
        if self.preload_metadata:
            self._entries[encoded_name] = key

        key.set_metadata('Content-Type', content_type)
        if isinstance(content, basestring):
            key.set_contents_from_string(
                content,
                headers=headers,
                policy=self.acl,
                reduced_redundancy=self.reduced_redundancy
            )
        else:
            content.name = cleaned_name
            key.set_contents_from_file(
                content,
                headers=headers,
                policy=self.acl,
                reduced_redundancy=self.reduced_redundancy
            )
        return self.open(encoded_name)

    def _open(self, name, mode='r'):
        return self.file_class(self, name=name, mode=mode)

    def delete_folder(self, name=None):
        if name is None:
            name = self.folder_name
        self.bucket.delete()

    def delete(self, name):
        name = self._encode_name(self._normalize_name(self._clean_name(name)))

        if self.bucket.lookup(name) is None:
            raise FileNotFoundError(name, 404)

        self.bucket.delete_key(name)

    def exists(self, name):
        name = self._normalize_name(self._clean_name(name))
        return bool(self.bucket.lookup(self._encode_name(name)))

    def url(self, name):
        name = self._normalize_name(self._clean_name(name))

        if self.custom_domain:
            return "%s://%s/%s" % ('https' if self.secure_urls else 'http',
                                   self.custom_domain, name)
        return self.connection.generate_url(
            self.querystring_expire,
            method='GET',
            bucket=self.bucket.name,
            key=self._encode_name(name),
            query_auth=self.querystring_auth,
            force_http=not self.secure_urls
        )

    @property
    def file_class(self):
        return S3BotoStorageFile


def require_opening(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        if not self._is_open:
            self._key.open(self._mode)
            self._is_open = True
        return func(self, *args, **kwargs)
    return wrapper


class S3BotoStorageFile(StorageFile):
    def __init__(self, storage, name=None, prefix='', mode='r'):
        if mode == 'rb':
            mode = 'r'  # rb is not supported

        self._storage = storage
        self.prefix = prefix
        self._key = Key(storage.bucket)
        self._mode = mode
        if name is not None:
            self.name = name
        self._pos = 0
        self._is_open = False

    @property
    def content_type(self):
        return getattr(
            self._key.content,
            'content_type',
            mimetypes.guess_type(self.name)[0] or Key.DefaultContentType
        )

    @property
    def file(self):
        return self._key

    @property
    @require_opening
    def size(self):
        return self._key.size

    @property
    @require_opening
    def last_modified(self):
        return self._key.last_modified

    @property
    def url(self):
        return self._storage.url(self._key.name)

    @StorageFile.name.setter
    def name(self, value):
        if self._name:
            raise StorageException(
                "You can't rename files this way. Use rename method instead."
            )
        self._name = self.prefix + self._storage._clean_name(value)
        self._key.name = self._name

    @require_opening
    def read(self, size=0):
        return self.file.read(size)

    def seek(self, *args, **kw):
        raise NotImplementedError

    def write(self, *args, **kw):
        raise NotImplementedError



import os
import itertools
from urlparse import urljoin

from .utils import force_str, force_unicode


__all__ = ('Storage')


def reraise(exception):
    kwargs = {
        'message': exception.message,
        'wrapped_exception': exception
    }
    # cloudfiles and S3Boto exceptions have http compatible status codes we
    # need to assign them to StorageException
    if hasattr(exception, 'status'):
        kwargs['status_code'] = exception.status

        if kwargs['status_code'] == 404:
            raise FileNotFoundError(**kwargs)
        elif kwargs['status_code'] == 409:
            raise FileExistsError(**kwargs)

    raise StorageException(**kwargs)


def safe_join(base, *paths):
    """
    A version of django.utils._os.safe_join for S3 paths.

    Joins one or more path components to the base path component
    intelligently. Returns a normalized version of the final path.

    The final path must be located inside of the base path component
    (otherwise a ValueError is raised).

    Paths outside the base path indicate a possible security
    sensitive operation.
    """
    base_path = force_unicode(base)
    base_path = base_path.rstrip('/')
    paths = [force_unicode(p) for p in paths]

    final_path = base_path
    for path in paths:
        final_path = urljoin(final_path.rstrip('/') + "/", path.rstrip("/"))

    # Ensure final_path starts with base_path and that the next character after
    # the final path is '/' (or nothing, in which case final_path must be
    # equal to base_path).
    base_path_len = len(base_path)
    if not final_path.startswith(base_path) \
            or final_path[base_path_len:base_path_len + 1] not in ('', '/'):
        raise ValueError('the joined path is located outside of the base path'
                         ' component')

    return final_path.lstrip('/')


class StorageException(Exception):
    def __init__(self, message='', status_code=None, wrapped_exception=None):
        self.status_code = status_code
        self.message = message
        self.wrapped_exception = wrapped_exception

    def __str__(self):
        return self.message


class FileNotFoundError(StorageException):
    pass


class ConflictError(StorageException):
    pass


class FileExistsError(StorageException):
    pass


class PermissionError(StorageException):
    pass


class RenameError(StorageException):
    pass


class Storage(object):
    """
    A base storage class, providing some default behaviors that all other
    storage systems can inherit or override, as necessary.
    """

    def open(self, name, mode='rb'):
        """
        Retrieves the specified file from storage.
        """
        return self._open(name, mode)

    def _open(self, name, mode='rb'):
        raise NotImplementedError

    def save(self, name, content, overwrite=False):
        """
        Saves new content to the file specified by name. The content should be
        a file-like object, ready to be read from the beginning.
        """
        name = os.path.normpath(name)

        if not overwrite:
            name = self.get_available_name(name)
        name = self._save(name, content)

        return name

    def _save(self, name, content):
        raise NotImplementedError

    def get_available_name(self, name):
        """
        Returns a filename that's free on the target storage system, and
        available for new content to be written to.
        """
        dir_name, file_name = os.path.split(name)
        file_root, file_ext = os.path.splitext(file_name)
        # If the filename already exists, add an underscore and a number
        # (before the file extension, if one exists) to the filename until the
        # generated filename doesn't exist.
        count = itertools.count(1)
        while self.exists(name):
            # file_ext includes the dot.
            newname = "%s_%s%s" % (file_root, count.next(), file_ext)
            name = safe_join(dir_name, newname)
        return name

    def path(self, name):
        """
        Returns a local filesystem path where the file can be retrieved using
        Python's built-in open() function. Storage systems that can't be
        accessed using open() should *not* implement this method.
        """
        raise NotImplementedError(
            "This backend doesn't support absolute paths."
        )

    def create_folder(self, name=None):
        """
        Tries to create given folder in the storage system.

        If no name is given tries to use the folder name of this object.

        The implementation of this method varies in subclasses:
            On rackspace cloudfiles this method creates a container.
            On amazon S3 this method creates a bucket.
            On normal filesystem this method creates a folder.
        """
        raise NotImplementedError

    def delete_folder(self, name=None):
        """
        Tries to delete given folder in the storage system.

        The implementation of this method varies in subclasses:
            On rackspace cloudfiles this method deletes a container.
            On amazon S3 this method deletes a bucket.
            On normal filesystem this method deletes a folder.
        """
        raise NotImplementedError

    def delete(self, name):
        """
        Deletes the specified file from the storage system.
        """
        raise NotImplementedError

    def exists(self, name):
        """
        Returns True if a file referened by the given name already exists in
        the storage system, or False if the name is available for a new file.
        """
        raise NotImplementedError

    def url(self, name):
        """
        Returns an absolute URL where the file's contents can be accessed
        directly by a Web browser.
        """
        raise NotImplementedError

    def _clean_name(self, name):
        """
        Cleans the name so that Windows style paths work
        """
        # Useful for windows' paths
        return os.path.normpath(name).replace('\\', '/')

    def _normalize_name(self, name):
        """
        Normalizes the name so that paths like
        /path/to/ignored/../something.txt
        work. We check to make sure that the path pointed to is not outside
        the directory specified by the LOCATION setting.
        """
        return safe_join(self.location, name)
        try:
            return safe_join(self.location, name)
        except ValueError:
            raise StorageException("Attempted access to '%s' denied." % name)

    def _encode_name(self, name):
        return force_str(name, self.file_name_charset)

    def _decode_name(self, name):
        return force_unicode(name, self.file_name_charset)

    def new_file(self, prefix=''):
        return self.file_class(self, prefix=prefix)

    def __eq__(self, other):
        if not isinstance(other, Storage):
            return NotImplemented
        return self.folder_name == other.folder_name

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result


class StorageFile(object):
    """
    Base class for driver file classes
    """
    _name = None
    prefix = ''
    _pos = 0

    @property
    def url(self):
        return self._storage.url(self._name)

    def delete(self):
        self._storage.delete(self._name)

    @property
    def storage(self):
        return self._storage

    @property
    def size(self):
        return self.file.size

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        if self._name:
            raise StorageException(
                "You can't rename files this way. Use rename method instead."
            )
        self._name = self.prefix + self._storage._clean_name(value)

    def rename(self, name):
        raise NotImplementedError

    def save(self, content, name=None):
        if name:
            self.name = name
        self._storage.save(self.name, content)

    def write(self, content):
        self._storage.save(self.name, content)

    def read(self, size=None):
        raise NotImplementedError

    def seek(self, offset, whence=os.SEEK_SET):
        if whence == os.SEEK_SET:
            self._pos = offset
        elif whence == os.SEEK_CUR:
            self._pos += offset
        elif whence == os.SEEK_END:
            self._pos = self.size + offset
        else:
            raise IOError(22, 'Invalid argument')

    def tell(self):
        return self._pos

    def __eq__(self, other):
        if not isinstance(other, StorageFile):
            return NotImplemented
        return (
            self.storage == other.storage and
            self.name == other.name
        )

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result

    def __nonzero__(self):
        return self._name is not None

    def __bool__(self):
        return self._name is not None



from .amazon import S3BotoStorage, S3BotoStorageFile
from .cloudfiles import CloudFilesStorage, CloudFilesStorageFile
from .filesystem import FileSystemStorage, FileSystemStorageFile
from .mock import MockStorage, MockStorageFile
from .base import (
    FileExistsError,
    FileNotFoundError,
    PermissionError,
    Storage,
    StorageException,
    StorageFile
)


__all__ = (
    CloudFilesStorage,
    CloudFilesStorageFile,
    FileExistsError,
    FileNotFoundError,
    FileSystemStorage,
    FileSystemStorageFile,
    MockStorage,
    MockStorageFile,
    PermissionError,
    S3BotoStorage,
    S3BotoStorageFile,
    Storage,
    StorageException,
    StorageFile,
    'STORAGE_DRIVERS',
    'get_default_storage_class',
    'get_filesystem_storage_class',
)


STORAGE_DRIVERS = {
    'amazon': S3BotoStorage,
    'cloudfiles': CloudFilesStorage,
    'filesystem': FileSystemStorage,
    'mock': MockStorage
}


def get_default_storage_class(app):
    return STORAGE_DRIVERS[app.config['DEFAULT_FILE_STORAGE']]


def get_filesystem_storage_class(app):
    if app.config['TESTING']:
        return MockStorage
    else:
        return FileSystemStorage



from __future__ import absolute_import

import mimetypes
import cloudfiles
from cloudfiles.errors import NoSuchObject, ResponseError, NoSuchContainer
from flask import current_app, request
from werkzeug.utils import cached_property

from .base import Storage, StorageFile, reraise

__all__ = ('CloudFilesStorage',)


class CloudFilesStorage(Storage):
    def __init__(self,
                 folder_name=None,
                 username=None,
                 api_key=None,
                 timeout=None):
        """
        Initialize the settings for the connection and container.
        """
        self.username = username or current_app.config.get(
            'CLOUDFILES_USERNAME', None)
        self.api_key = api_key or current_app.config.get(
            'CLOUDFILES_API_KEY', None)
        self.container_name = folder_name or \
            current_app.config.get('CLOUDFILES_CONTAINER', None)
        self.timeout = timeout or current_app.config.get(
            'CLOUDFILES_TIMEOUT', 5)
        self.use_servicenet = current_app.config.get(
            'CLOUDFILES_SERVICENET', False)
        self.auto_create_container = current_app.config.get(
            'CLOUDFILES_AUTO_CREATE_CONTAINER', False)
        self.secure_uris = current_app.config.get(
            'CLOUDFILES_SECURE_URIS', False)

    @property
    def folder_name(self):
        return self.container_name

    @property
    def folder(self):
        return self.container

    @cached_property
    def connection(self):
        return cloudfiles.get_connection(
            username=self.username,
            api_key=self.api_key,
            timeout=self.timeout,
            servicenet=self.use_servicenet
        )

    @property
    def container(self):
        if not hasattr(self, '_container'):
            self._container = self._get_or_create_container(self.container_name)
        if not self._container.is_public():
            self._container.make_public()
        return self._container

    @cached_property
    def container_url(self):
        container_uris = current_app.config.get(
            'CLOUDFILES_CONTAINER_URIS', {})
        if self.container_name in container_uris:
            return container_uris[self.container_name]
        if self.secure_uris or request.is_secure:
            return self.container.public_ssl_uri()
        else:
            return self.container.public_uri()

    def _get_or_create_container(self, name):
        """Retrieves a bucket if it exists, otherwise creates it."""
        try:
            return self.connection.get_container(name)
        except NoSuchContainer:
            if self.auto_create_container:
                return self.connection.create_container(name)
            else:
                raise RuntimeError(
                    "Container specified by "
                    "CLOUDFILES_BUCKET_NAME does not exist. "
                    "Containers can be automatically created by setting "
                    "CLOUDFILES_AUTO_CREATE_CONTAINER=True")

    def _save(self, name, content):
        """
        Use the Cloud Files service to write a `werkzeug.FileStorage`
        (called ``file``) to a remote file (called ``name``).
        """
        cloud_obj = self.container.create_object(name)
        mimetype, _ = mimetypes.guess_type(name)
        cloud_obj.content_type = mimetype
        cloud_obj.send(content)
        return self.open(name)

    def _open(self, name, mode='rb'):
        return self.file_class(self, name)

    def delete(self, name):
        """
        Deletes the specified file from the storage system.
        """
        try:
            self.container.delete_object(name)
        except ResponseError, e:
            reraise(e)

    def exists(self, name):
        """
        Returns True if a file referenced by the given name already exists in
        the storage system, or False if the name is available for a new file.
        """
        try:
            self.container.get_object(name)
            return True
        except NoSuchObject:
            return False

    def url(self, name):
        """
        Returns an absolute URL where the file's contents can be accessed
        directly by a web browser.
        """
        return '%s/%s' % (self.container_url, name)

    def get_object(self, name):
        try:
            return self.container.get_object(name)
        except NoSuchObject, e:
            reraise(e)
        except ResponseError, e:
            reraise(e)

    @property
    def file_class(self):
        return CloudFilesStorageFile


class CloudFilesStorageFile(StorageFile):
    _file = None

    def __init__(self, storage, name=None, prefix=''):
        self._storage = storage
        self.prefix = prefix
        if name is not None:
            self.name = name
        if self._name:
            self.file
        self._pos = 0

    @property
    def file(self):
        if not self._file:
            self._file = self._storage.get_object(self.name)
        return self._file

    def read(self, size=-1, **kw):
        kw['offset'] = self._pos
        data = self.file.read(size, **kw)
        self._pos += len(data)
        return data



from setuptools import setup, Command
import subprocess


class PyTest(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        errno = subprocess.call(['py.test'])
        raise SystemExit(errno)


setup(
    name='Flask-Storage-Helpers',
    version='0.1.4',
    url='http://github.com/achauve/flask-storage-helpers',
    license='MIT',
    author='Konsta Vesterinen',
    author_email='konsta.vesterinen@gmail.com',
    description='Various file storage backends for Flask apps.',
    long_description=
        open('README.rst').read() + '\n\n' +
        open('CHANGES.rst').read(),
    packages=['flask_storage_helpers'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask>=0.7',
        'boto>=2.5.2',
        'python-cloudfiles>=1.7.10'
    ],
    cmdclass={'test': PyTest},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



def force_str(name, encoding='utf-8'):
    if isinstance(name, str):
        return name
    else:
        return name.encode(encoding)


def force_unicode(name, encoding='utf-8'):
    if isinstance(name, unicode):
        return name
    else:
        return name.decode(encoding)



import os
from datetime import datetime
from .base import Storage, StorageFile, FileNotFoundError


class MockStorage(Storage):
    """
    A mock storage class for testing
    """
    _files = {}

    def __init__(self, folder_name=''):
        self.folder_name = folder_name

    def _save(self, name, content):
        if isinstance(content, basestring):
            self._files[name] = content
        else:
            content.seek(0)
            self._files[name] = content.read()
        return self.open(name)

    def _open(self, name, mode):
        return self.file_class(self, name)

    def path(self, name):
        """
        Returns a local filesystem path where the file can be retrieved using
        Python's built-in open() function. Storage systems that can't be
        accessed using open() should *not* implement this method.
        """

    def delete(self, name):
        """
        Deletes the specified file from the storage system.
        """
        try:
            del self._files[name]
        except KeyError:
            raise FileNotFoundError()

    def exists(self, name):
        """
        Returns True if a file referened by the given name already exists in
        the storage system, or False if the name is available for a new file.
        """
        return name in self._files

    def url(self, name):
        """
        Returns an absolute URL where the file's contents can be accessed
        directly by a Web browser.
        """
        return os.path.join(self.folder_name, name)

    def empty(self):
        self._files.clear()

    @property
    def file_class(self):
        return MockStorageFile


class MockStorageFile(StorageFile):
    def __init__(self, storage, name=None, prefix=''):
        self._storage = storage
        if name is not None:
            self.name = name

        self.prefix = prefix

        if self.name:
            self.file
        self._pos = 0
        self.last_modified = datetime.now()

    def rename(self, name):
        self._name = name

    @property
    def file(self):
        try:
            return self._storage._files[self.name]
        except KeyError:
            raise FileNotFoundError()

    @property
    def size(self):
        return len(self.file)

    def read(self, size=-1):
        if size < 0:
            size = self.size

        start = self._pos
        end = min(self.size, self._pos + size)
        self._pos = end
        return self.file[start:end]
